@@ -47,7 +47,7 @@ class TionDummy:
     _dummy_data: bytearray
 
     @staticmethod
-    def _connect_dummy():
+    def _connect_dummy(need_notifications: bool = True):
         """dummy connection"""
 
         _LOGGER.info("Dummy connect")
@@ -96,6 +96,7 @@ class tion(TionDummy):
         self._heating: bool = False
         self._filter_remain: float = 0.0
         self._error_code: int = 0
+        self.__failed_connects: int = 0
 
         if self.mac == "dummy":
             _LOGGER.warning("Dummy mode detected!")
@@ -294,7 +295,8 @@ class tion(TionDummy):
 
         return connection_status
 
-    def _connect(self):
+    def _connect(self, need_notifications: bool = True):
+        _LOGGER.debug("Connecting")
         if self.connection_status == "disc":
             try:
                 self._btle.connect(self.mac, btle.ADDR_TYPE_RANDOM)
@@ -303,12 +305,20 @@ class tion(TionDummy):
                         self.notify = tc
                     if tc.uuid == self.uuid_write:
                         self.write = tc
-
-                self._enable_notifications()
+                if need_notifications:
+                    self._enable_notifications()
+                else:
+                    _LOGGER.debug("Notifications was not requested")
+                self.__failed_connects = 0
             except btle.BTLEDisconnectError as e:
-                _LOGGER.warning("Got %s exception", str(e))
-                time.sleep(2)
-                raise e
+                _LOGGER.warning("Got BTLEDisconnectError:%s", str(e))
+                if self.__failed_connects < 1:
+                    self.__failed_connects += 1
+                    _LOGGER.debug("Will try again.")
+                    time.sleep(2)
+                    self._connect(need_notifications)
+                else:
+                    raise e
 
     def _disconnect(self):
         if self.connection_status != "disc":
@@ -345,13 +355,22 @@ class tion(TionDummy):
 
     def _enable_notifications(self):
         _LOGGER.debug("Enabling notification")
+        setup_data = b"\x01\x00"
+
+        _LOGGER.debug("Notify handler is %s", self.notify.getHandle())
+        notify_handle = self.notify.getHandle() + 1
+
+        _LOGGER.debug("Will write %s to %s handle", setup_data, notify_handle)
+        result = self._btle.writeCharacteristic(notify_handle, setup_data, withResponse=True)
+        _LOGGER.debug("Result is %s", result)
         self._btle.withDelegate(self._delegation)
         _LOGGER.debug("Delegation enabled")
         try:
-            self.notify.read()
-            _LOGGER.debug("First read done")
-        except Exception as e:
-            _LOGGER.warning("Got exception '%s' while first read, but it is OK and we may ignore it", str(e))
+            data = self.notify.read()
+            _LOGGER.debug("First read done. Data is %s", bytes(data).hex())
+        except btle.BTLEDisconnectError as e:
+            _LOGGER.critical("_enable_notifications: got '%s' while first read! Could not continue!", str(e))
+            raise e
 
         self._delegation.setReadTopic(self.notify)
         _LOGGER.debug("enable_notification is done")
@@ -466,7 +485,7 @@ class tion(TionDummy):
 
     def pair(self):
         _LOGGER.debug("Pairing")
-        self._connect()
+        self._connect(need_notifications=False)
         _LOGGER.debug("Connected. BT pairing ...")
         try:
             # use private methods to avoid disconnect if already paired
@@ -478,8 +497,13 @@ class tion(TionDummy):
             except KeyError:
                 # we may have no estat in response. It is OK.
                 estat = 0
-            # 0 -- fine, 19 -- paired.
-            if estat == 0 or estat == 19 or rsp['code'][0] == 'success':
+
+            if estat == 20:
+                # it seem in https://github.com/TionAPI/tion_python/issues/17 that we may ignore it.
+                _LOGGER.warning("bt pairing: could not pair! Permission denied. Check permissions at host. "
+                                "Ignore it if all goes well")
+            elif estat == 0 or estat == 19 or rsp['code'][0] == 'success':
+                # 0 -- fine, no errors; 19 -- paired.
                 try:
                     msg = rsp['emsg'][0]
                 except KeyError:
